#!/bin/bash
# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.

set -e
set -x

# Time marker for both stderr and stdout
date; date 1>&2

# More conservative umask for logs, indexes, and such.
umask 027

# BASE_JAVA_OPTS may be overwritten
BASE_JAVA_OPTS="-server -XX:+UseConcMarkSweepGC -XX:-CMSConcurrentMTEnabled -XX:+UseParNewGC"

MGMT_JAVA_OPTS="$BASE_JAVA_OPTS $MGMT_JAVA_OPTS"
MGMT_JAVA_OPTS="$MGMT_JAVA_OPTS -Dmgmt.log.file=$MGMT_LOG_FILE"
MGMT_JAVA_OPTS="$MGMT_JAVA_OPTS -Djava.awt.headless=true"
MGMT_JAVA_OPTS="$MGMT_JAVA_OPTS -Djava.net.preferIPv4Stack=true"

# This is the name of the command line argument we pass to the
# firehose and headlamp services when we start them. This must be kept in
# sync with com.cloudera.cmon.Firehose.Main and
# com.cloudera.headlamp.HeadlampConfiguration classes.
MGMT_HOME_ARG_NAME="--mgmt-home"

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

# attempt to find java
locate_java_home

# Load the parcel environment
source_parcel_environment

if [ "$JAVA_HOME" != "" ]; then
  JAVA="$JAVA_HOME/bin/java"
fi

if [[ "$MGMT_HOME" ]]; then
  # "*" is interpreted by Java, to mean all jars in dir.
  MGMT_CLASSPATH="$MGMT_HOME/lib/*"
fi

if [[ -z "$MGMT_CLASSPATH" ]]; then
  echo "\$MGMT_CLASSPATH" is not set.
fi

# Add JDBC jars to the path. Oracle and Mysql installed by end user, can't ship due to licensing.
JDBC_JARS="$CLOUDERA_MYSQL_CONNECTOR_JAR:$CLOUDERA_POSTGRESQL_JDBC_JAR:$CLOUDERA_ORACLE_CONNECTOR_JAR"
MGMT_CLASSPATH="${JDBC_JARS}:${MGMT_CLASSPATH}"

DDL_DIR="$MGMT_HOME/schema"

if [[ "$1" == "firehose" ]]; then
  shift

  MGMT_JAVA_OPTS="$MGMT_JAVA_OPTS -Dfirehose.schema.dir=$DDL_DIR"
  # We set the connection and responseTimeout for SMON to make sure its calls as
  # a JMX client have appropriate connection and read timeouts. To adjust these,
  # the additional JVM options configuration parameter can be used, as those
  # values will be last and will therefore take effect.
  # 
  # See http://docs.oracle.com/javase/6/docs/technotes/guides/rmi/sunrmiproperties.html
  # for the documentation of these options.
  #
  # We also give SMON more permgen than the default value.
  if [[ $@ =~ "SERVICE_MONITORING" ]]; then
    MGMT_JAVA_OPTS="$MGMT_JAVA_OPTS -XX:PermSize=128m -Dsun.rmi.transport.tcp.handshakeTimeout=10000 -Dsun.rmi.transport.tcp.responseTimeout=10000"
  fi

  # Current directory has the config file and the log4j file
  MGMT_CLASSPATH="$(pwd):$MGMT_CLASSPATH"

  # We need to add the MGMT_HOME directory as an argument to the jvm.
  echo Executing: ${JAVA} ${MGMT_JAVA_OPTS} ${FIREHOSE_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.cmon.firehose.Main $@ "${MGMT_HOME_ARG_NAME}" "${MGMT_HOME}"
  exec ${JAVA} ${MGMT_JAVA_OPTS} ${FIREHOSE_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.cmon.firehose.Main "$@" "${MGMT_HOME_ARG_NAME}" "${MGMT_HOME}"

elif [[ "$1" == "eventserver" ]]; then
  shift

  # Current directory has the config file and the log4j file
  MGMT_CLASSPATH="$(pwd):$MGMT_CLASSPATH"

  echo Executing: ${JAVA} ${MGMT_JAVA_OPTS} ${EVENTSERVER_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.cmf.eventcatcher.server.EventCatcherService "$@"
  exec ${JAVA} ${MGMT_JAVA_OPTS} ${EVENTSERVER_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.cmf.eventcatcher.server.EventCatcherService "$@"

elif [[ "$1" == "alertpublisher" ]]; then
  shift

  # Current directory has the config file and the log4j file
  MGMT_CLASSPATH="$(pwd):$MGMT_CLASSPATH"

  echo "Executing: " ${JAVA} ${MGMT_JAVA_OPTS} ${ALERTPUBLISHER_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.enterprise.alertpublisher.AlertPublisher "$@"
  exec ${JAVA} ${MGMT_JAVA_OPTS} ${ALERTPUBLISHER_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.enterprise.alertpublisher.AlertPublisher "$@"

elif [[ "$1" == "headlamp" ]]; then
  shift

  MGMT_JAVA_OPTS="$MGMT_JAVA_OPTS -Dheadlamp.schema.dir=$DDL_DIR"

  # Current directory has the config file and the log4j file
  MGMT_CLASSPATH="$(pwd):$MGMT_CLASSPATH"

  echo "Executing: " ${JAVA} ${MGMT_JAVA_OPTS} ${HEADLAMP_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.headlamp.HeadlampServer --db-conf-dir "`pwd`" --db-conf-file headlamp.db.properties "$@" "${MGMT_HOME_ARG_NAME}" "${MGMT_HOME}"
  exec ${JAVA} ${MGMT_JAVA_OPTS} ${HEADLAMP_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.headlamp.HeadlampServer --db-conf-dir "`pwd`" --db-conf-file headlamp.db.properties "$@" "${MGMT_HOME_ARG_NAME}" "${MGMT_HOME}"
elif [[ "$1" == "inspector" ]]; then
  shift

  # Current directory has the config file and the log4j file
  MGMT_CLASSPATH="$(pwd):$MGMT_CLASSPATH"

  echo "Executing: " ${JAVA} ${MGMT_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.cmf.Inspector "$@"
  exec ${JAVA} ${MGMT_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.cmf.inspector.Inspector "$@"

elif [[ "$1" == "navigator" ]]; then
  shift

  MGMT_JAVA_OPTS="$MGMT_JAVA_OPTS -Dnavigator.schema.dir=$DDL_DIR"

  # Current directory has the config file and the log4j file
  MGMT_CLASSPATH="$(pwd):$MGMT_CLASSPATH"

  echo "Executing: " ${JAVA} ${MGMT_JAVA_OPTS} ${NAVIGATOR_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.navigator.NavigatorMain "$@" --conf-dir "`pwd`"
  exec ${JAVA} ${MGMT_JAVA_OPTS} ${NAVIGATOR_JAVA_OPTS} -cp "${MGMT_CLASSPATH}" com.cloudera.navigator.NavigatorMain "$@" --conf-dir "`pwd`"

else
  echo "Unsupported arguments: $*"
  exit 1
fi
